# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "LibraryScreenStartResponse",
    "Error",
    "Input",
    "InputProteins",
    "InputTarget",
    "InputTargetStructureTemplateTargetResponse",
    "InputTargetStructureTemplateTargetResponseChainSelection",
    "InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetPolymerChainSpec",
    "InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetLigandChainSpec",
    "InputTargetStructureTemplateTargetResponseStructure",
    "InputTargetNoTemplateTargetResponse",
    "InputTargetNoTemplateTargetResponseEntity",
    "InputTargetNoTemplateTargetResponseEntityProteinEntityResponse",
    "InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModification",
    "InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationCcdModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationSmilesModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityRnaEntityResponse",
    "InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModification",
    "InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationCcdModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationSmilesModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityDnaEntityResponse",
    "InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModification",
    "InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationCcdModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationSmilesModificationResponse",
    "InputTargetNoTemplateTargetResponseEntityLigandCcdEntityResponse",
    "InputTargetNoTemplateTargetResponseEntityLigandSmilesEntityResponse",
    "InputTargetNoTemplateTargetResponseBond",
    "InputTargetNoTemplateTargetResponseBondAtom1",
    "InputTargetNoTemplateTargetResponseBondAtom1LigandAtomResponse",
    "InputTargetNoTemplateTargetResponseBondAtom1PolymerAtomResponse",
    "InputTargetNoTemplateTargetResponseBondAtom2",
    "InputTargetNoTemplateTargetResponseBondAtom2LigandAtomResponse",
    "InputTargetNoTemplateTargetResponseBondAtom2PolymerAtomResponse",
    "InputTargetNoTemplateTargetResponseConstraint",
    "InputTargetNoTemplateTargetResponseConstraintPocketConstraintResponse",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponse",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1PolymerContactTokenResponse",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1LigandContactTokenResponse",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2PolymerContactTokenResponse",
    "InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2LigandContactTokenResponse",
    "Progress",
]


class Error(BaseModel):
    code: str
    """Machine-readable error code"""

    message: str
    """Human-readable error message"""

    details: Optional[object] = None
    """Additional error details"""


class InputProteins(BaseModel):
    url: str
    """URL to download the file"""

    url_expires_at: datetime
    """When the presigned URL expires"""


class InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetPolymerChainSpec(BaseModel):
    """
    Per-chain specification for a polymer (protein/RNA/DNA) chain in a structure template target.
    """

    chain_type: Literal["polymer"]

    crop_residues: Union[List[int], Literal["all"]]
    """
    0-indexed residue indices to retain from this chain, or 'all' to keep all
    residues. Residues not listed are excluded from the engine run.
    """

    epitope_residues: Optional[List[int]] = None
    """0-indexed residue indices where binder contact is desired (the epitope).

    All indices must be present in crop_residues.
    """

    flexible_residues: Optional[List[int]] = None
    """0-indexed residue indices allowed to move during design (e.g.

    flexible loop regions). All indices must be present in crop_residues.
    """


class InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetLigandChainSpec(BaseModel):
    """Per-chain specification for a ligand chain in a structure template target.

    The full ligand is always included.
    """

    chain_type: Literal["ligand"]


InputTargetStructureTemplateTargetResponseChainSelection: TypeAlias = Union[
    InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetPolymerChainSpec,
    InputTargetStructureTemplateTargetResponseChainSelectionStructureTemplateTargetLigandChainSpec,
]


class InputTargetStructureTemplateTargetResponseStructure(BaseModel):
    url: str
    """URL to download the file"""

    url_expires_at: datetime
    """When the presigned URL expires"""


class InputTargetStructureTemplateTargetResponse(BaseModel):
    """Target defined by an uploaded 3D structure (CIF or PDB file).

    Only chains included in chain_selection are used.
    """

    chain_selection: Dict[str, InputTargetStructureTemplateTargetResponseChainSelection]
    """Chains selected from the uploaded structure, keyed by chain ID.

    Only chains listed here are included in the engine run — any chains omitted from
    this mapping are ignored. Each value defines which residues to keep, which are
    epitope residues, and which are flexible.
    """

    structure: InputTargetStructureTemplateTargetResponseStructure

    type: Literal["structure_template"]


class InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationCcdModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["ccd"]

    value: str
    """CCD code from RCSB PDB (e.g.

    'MSE' for selenomethionine, 'SEP' for phosphoserine)
    """


class InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationSmilesModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["smiles"]

    value: str
    """SMILES string for the modification"""


InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModification: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationCcdModificationResponse,
    InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModificationSmilesModificationResponse,
]


class InputTargetNoTemplateTargetResponseEntityProteinEntityResponse(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this entity"""

    modifications: List[InputTargetNoTemplateTargetResponseEntityProteinEntityResponseModification]
    """Post-translational modifications"""

    type: Literal["protein"]

    value: str
    """Amino acid sequence (one-letter codes)"""

    cyclic: Optional[bool] = None
    """Whether the sequence is cyclic"""


class InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationCcdModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["ccd"]

    value: str
    """CCD code from RCSB PDB (e.g.

    'MSE' for selenomethionine, 'SEP' for phosphoserine)
    """


class InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationSmilesModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["smiles"]

    value: str
    """SMILES string for the modification"""


InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModification: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationCcdModificationResponse,
    InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModificationSmilesModificationResponse,
]


class InputTargetNoTemplateTargetResponseEntityRnaEntityResponse(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this entity"""

    modifications: List[InputTargetNoTemplateTargetResponseEntityRnaEntityResponseModification]
    """Chemical modifications"""

    type: Literal["rna"]

    value: str
    """RNA nucleotide sequence (A, C, G, U, N)"""

    cyclic: Optional[bool] = None
    """Whether the sequence is cyclic"""


class InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationCcdModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["ccd"]

    value: str
    """CCD code from RCSB PDB (e.g.

    'MSE' for selenomethionine, 'SEP' for phosphoserine)
    """


class InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationSmilesModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["smiles"]

    value: str
    """SMILES string for the modification"""


InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModification: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationCcdModificationResponse,
    InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModificationSmilesModificationResponse,
]


class InputTargetNoTemplateTargetResponseEntityDnaEntityResponse(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this entity"""

    modifications: List[InputTargetNoTemplateTargetResponseEntityDnaEntityResponseModification]
    """Chemical modifications"""

    type: Literal["dna"]

    value: str
    """DNA nucleotide sequence (A, C, G, T, N)"""

    cyclic: Optional[bool] = None
    """Whether the sequence is cyclic"""


class InputTargetNoTemplateTargetResponseEntityLigandCcdEntityResponse(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this ligand"""

    type: Literal["ligand_ccd"]

    value: str
    """CCD code (e.g., ATP, ADP)"""


class InputTargetNoTemplateTargetResponseEntityLigandSmilesEntityResponse(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this ligand"""

    type: Literal["ligand_smiles"]

    value: str
    """SMILES string representing the ligand"""


InputTargetNoTemplateTargetResponseEntity: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseEntityProteinEntityResponse,
    InputTargetNoTemplateTargetResponseEntityRnaEntityResponse,
    InputTargetNoTemplateTargetResponseEntityDnaEntityResponse,
    InputTargetNoTemplateTargetResponseEntityLigandCcdEntityResponse,
    InputTargetNoTemplateTargetResponseEntityLigandSmilesEntityResponse,
]


class InputTargetNoTemplateTargetResponseBondAtom1LigandAtomResponse(BaseModel):
    atom_name: str
    """Standardized atom name (verifiable in CIF file on RCSB)"""

    chain_id: str
    """Chain ID containing the atom"""

    type: Literal["ligand_atom"]


class InputTargetNoTemplateTargetResponseBondAtom1PolymerAtomResponse(BaseModel):
    atom_name: str
    """Standardized atom name (verifiable in CIF file on RCSB)"""

    chain_id: str
    """Chain ID containing the atom"""

    residue_index: int
    """0-based residue index"""

    type: Literal["polymer_atom"]


InputTargetNoTemplateTargetResponseBondAtom1: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseBondAtom1LigandAtomResponse,
    InputTargetNoTemplateTargetResponseBondAtom1PolymerAtomResponse,
]


class InputTargetNoTemplateTargetResponseBondAtom2LigandAtomResponse(BaseModel):
    atom_name: str
    """Standardized atom name (verifiable in CIF file on RCSB)"""

    chain_id: str
    """Chain ID containing the atom"""

    type: Literal["ligand_atom"]


class InputTargetNoTemplateTargetResponseBondAtom2PolymerAtomResponse(BaseModel):
    atom_name: str
    """Standardized atom name (verifiable in CIF file on RCSB)"""

    chain_id: str
    """Chain ID containing the atom"""

    residue_index: int
    """0-based residue index"""

    type: Literal["polymer_atom"]


InputTargetNoTemplateTargetResponseBondAtom2: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseBondAtom2LigandAtomResponse,
    InputTargetNoTemplateTargetResponseBondAtom2PolymerAtomResponse,
]


class InputTargetNoTemplateTargetResponseBond(BaseModel):
    atom1: InputTargetNoTemplateTargetResponseBondAtom1

    atom2: InputTargetNoTemplateTargetResponseBondAtom2


class InputTargetNoTemplateTargetResponseConstraintPocketConstraintResponse(BaseModel):
    """Constrains the binder to interact with specific pocket residues on the target."""

    binder_chain_id: str
    """Chain ID of the binder molecule"""

    contact_residues: Dict[str, List[int]]
    """Binding pocket residues keyed by chain ID.

    Each key is a chain ID (e.g. "A") and the value is an array of 0-indexed residue
    indices that define the pocket on that chain.
    """

    max_distance_angstrom: float
    """Maximum allowed distance in Angstroms between binder and pocket residues.

    Typical range: 4-8 A.
    """

    type: Literal["pocket"]

    force: Optional[bool] = None
    """Whether to force the constraint"""


class InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1PolymerContactTokenResponse(
    BaseModel
):
    chain_id: str
    """Chain ID"""

    residue_index: int
    """0-based residue index"""

    type: Literal["polymer_contact"]


class InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1LigandContactTokenResponse(BaseModel):
    atom_name: str
    """Atom name"""

    chain_id: str
    """Chain ID"""

    type: Literal["ligand_contact"]


InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1PolymerContactTokenResponse,
    InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1LigandContactTokenResponse,
]


class InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2PolymerContactTokenResponse(
    BaseModel
):
    chain_id: str
    """Chain ID"""

    residue_index: int
    """0-based residue index"""

    type: Literal["polymer_contact"]


class InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2LigandContactTokenResponse(BaseModel):
    atom_name: str
    """Atom name"""

    chain_id: str
    """Chain ID"""

    type: Literal["ligand_contact"]


InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2PolymerContactTokenResponse,
    InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2LigandContactTokenResponse,
]


class InputTargetNoTemplateTargetResponseConstraintContactConstraintResponse(BaseModel):
    max_distance_angstrom: float
    """Maximum distance in Angstroms"""

    token1: InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken1

    token2: InputTargetNoTemplateTargetResponseConstraintContactConstraintResponseToken2

    type: Literal["contact"]

    force: Optional[bool] = None
    """Whether to force the constraint"""


InputTargetNoTemplateTargetResponseConstraint: TypeAlias = Union[
    InputTargetNoTemplateTargetResponseConstraintPocketConstraintResponse,
    InputTargetNoTemplateTargetResponseConstraintContactConstraintResponse,
]


class InputTargetNoTemplateTargetResponse(BaseModel):
    """Target defined by sequences only, without a 3D structure template"""

    entities: List[InputTargetNoTemplateTargetResponseEntity]
    """Entities (proteins, RNA, DNA, ligands) defining the target complex."""

    type: Literal["no_template"]

    bonds: Optional[List[InputTargetNoTemplateTargetResponseBond]] = None
    """Covalent bond constraints between atoms in the target complex."""

    constraints: Optional[List[InputTargetNoTemplateTargetResponseConstraint]] = None
    """Structural constraints (pocket and contact)"""

    epitope_residues: Optional[Dict[str, List[int]]] = None
    """Residues where binder contact is desired (the epitope).

    Each key is a chain ID and the value is an array of 0-indexed residue indices.
    All indices must be valid within the corresponding entity sequence.
    """


InputTarget: TypeAlias = Union[InputTargetStructureTemplateTargetResponse, InputTargetNoTemplateTargetResponse]


class Input(BaseModel):
    proteins: InputProteins

    target: InputTarget
    """Target specification (structure template or template-free)"""


class Progress(BaseModel):
    num_proteins_failed: int
    """Number of accepted proteins that reached terminal failure during screening."""

    num_proteins_screened: int
    """Number of accepted proteins that produced usable screening results."""

    total_proteins_to_screen: int
    """Total number of proteins accepted into the screening run."""

    latest_result_id: Optional[str] = None
    """ID of the latest result"""


class LibraryScreenStartResponse(BaseModel):
    """A protein library screening engine run"""

    id: str
    """Unique ProteinLibraryScreen identifier"""

    completed_at: Optional[datetime] = None

    created_at: datetime

    data_deleted_at: Optional[datetime] = None
    """When the input, output, and result data was permanently deleted.

    Null if data has not been deleted.
    """

    engine: Literal["boltz-protein-screen"]
    """Engine used for protein library screen"""

    engine_version: str
    """Engine version used for protein library screen"""

    error: Optional[Error] = None

    input: Input

    livemode: bool
    """Whether this resource was created with a live API key."""

    progress: Optional[Progress] = None

    started_at: Optional[datetime] = None

    status: Literal["pending", "running", "succeeded", "failed", "stopped"]

    stopped_at: Optional[datetime] = None

    workspace_id: str
    """Workspace ID"""

    idempotency_key: Optional[str] = None
    """Client-provided idempotency key"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["DesignListResultsResponse", "Artifacts", "ArtifactsArchive", "ArtifactsStructure", "Metrics", "Warning"]


class ArtifactsArchive(BaseModel):
    url: str
    """URL to download the file"""

    url_expires_at: datetime
    """When the presigned URL expires"""


class ArtifactsStructure(BaseModel):
    url: str
    """URL to download the file"""

    url_expires_at: datetime
    """When the presigned URL expires"""


class Artifacts(BaseModel):
    archive: ArtifactsArchive

    structure: ArtifactsStructure


class Metrics(BaseModel):
    """Scoring metrics for a designed small molecule"""

    binding_confidence: float
    """Confidence that the molecule binds the target (0-1).

    Primary metric for hit discovery.
    """

    complex_iplddt: float
    """Interface pLDDT for the complex (0-100). Confidence at the binding interface."""

    complex_plddt: float
    """pLDDT for the full complex (0-100)."""

    iptm: float
    """Interface predicted TM score (0-1).

    Confidence in relative positioning of ligand and protein.
    """

    optimization_score: float
    """Binding strength ranking score for lead optimization.

    Higher values indicate stronger predicted binding.
    """

    plddt: float
    """Predicted local distance difference test score (0-100).

    Per-residue structure confidence.
    """

    ptm: float
    """Predicted TM score (0-1). Global structure quality metric."""

    structure_confidence: float
    """Confidence in the predicted 3D structure (0-1)."""


class Warning(BaseModel):
    """A warning about a potential quality issue with a result"""

    code: str
    """Machine-readable warning code (e.g. "low_confidence", "unusual_geometry")"""

    message: str
    """Human-readable description of the warning"""


class DesignListResultsResponse(BaseModel):
    """A single designed small molecule result"""

    id: str
    """Unique result ID"""

    artifacts: Artifacts

    created_at: datetime

    metrics: Metrics
    """Scoring metrics for a designed small molecule"""

    smiles: str
    """SMILES string of the designed molecule"""

    warnings: Optional[List[Warning]] = None
    """Warnings about potential quality issues with this result."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DesignDeleteDataResponse"]


class DesignDeleteDataResponse(BaseModel):
    id: str
    """ID of the resource whose data was deleted"""

    data_deleted: Literal[True]

    data_deleted_at: datetime
    """When the data was deleted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "DesignStopResponse",
    "Error",
    "Input",
    "InputTarget",
    "InputTargetEntity",
    "InputTargetEntityModification",
    "InputTargetEntityModificationCcdModificationResponse",
    "InputTargetEntityModificationSmilesModificationResponse",
    "InputMoleculeFilters",
    "InputMoleculeFiltersCustomFilter",
    "InputMoleculeFiltersCustomFilterLipinskiFilterResponse",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponse",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseFractionCsp3",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolLogp",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolWt",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumAromaticRings",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHAcceptors",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHDonors",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHeteroatoms",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRings",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRotatableBonds",
    "InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseTpsa",
    "InputMoleculeFiltersCustomFilterSmartsCustomFilterResponse",
    "InputMoleculeFiltersCustomFilterSmartsCatalogFilterResponse",
    "InputMoleculeFiltersCustomFilterSmilesRegexFilterResponse",
    "Progress",
]


class Error(BaseModel):
    code: str
    """Machine-readable error code"""

    message: str
    """Human-readable error message"""

    details: Optional[object] = None
    """Additional error details"""


class InputTargetEntityModificationCcdModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["ccd"]

    value: str
    """CCD code from RCSB PDB (e.g.

    'MSE' for selenomethionine, 'SEP' for phosphoserine)
    """


class InputTargetEntityModificationSmilesModificationResponse(BaseModel):
    residue_index: int
    """0-based index of the residue to modify"""

    type: Literal["smiles"]

    value: str
    """SMILES string for the modification"""


InputTargetEntityModification: TypeAlias = Union[
    InputTargetEntityModificationCcdModificationResponse, InputTargetEntityModificationSmilesModificationResponse
]


class InputTargetEntity(BaseModel):
    chain_ids: List[str]
    """Chain IDs for this entity"""

    modifications: List[InputTargetEntityModification]
    """Post-translational modifications"""

    type: Literal["protein"]

    value: str
    """Amino acid sequence (one-letter codes)"""

    cyclic: Optional[bool] = None
    """Whether the sequence is cyclic"""


class InputTarget(BaseModel):
    """Target protein with binding pocket for small molecule design or screening"""

    entities: List[InputTargetEntity]
    """Protein entities defining the target structure.

    Each entity represents a protein chain.
    """

    pocket_residues: Optional[Dict[str, List[int]]] = None
    """Binding pocket residues, keyed by chain ID.

    Each key is a chain ID (e.g. "A") and the value is an array of 0-indexed residue
    indices that define the binding pocket on that chain. When provided, these
    residues guide pocket extraction. When omitted, the model auto-detects the
    pocket.
    """

    reference_ligands: Optional[List[str]] = None
    """
    Reference ligands as SMILES strings that help the model identify the binding
    pocket. When omitted, a set of drug-like default ligands is used for pocket
    detection.
    """


class InputMoleculeFiltersCustomFilterLipinskiFilterResponse(BaseModel):
    """Lipinski's Rule of Five filter.

    Rejects molecules that violate drug-likeness criteria based on molecular weight, LogP, hydrogen bond donors, and hydrogen bond acceptors.
    """

    max_hba: float
    """Maximum number of hydrogen bond acceptors. Lipinski threshold: 10"""

    max_hbd: float
    """Maximum number of hydrogen bond donors. Lipinski threshold: 5"""

    max_logp: float
    """Maximum LogP. Lipinski threshold: 5"""

    max_mw: float
    """Maximum molecular weight (Da). Lipinski threshold: 500"""

    type: Literal["lipinski_filter"]

    allow_single_violation: Optional[bool] = None
    """If true, one rule violation is allowed (classic Rule of Five).

    Defaults to false (all rules must pass).
    """


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseFractionCsp3(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolLogp(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolWt(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumAromaticRings(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHAcceptors(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHDonors(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHeteroatoms(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRings(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRotatableBonds(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseTpsa(BaseModel):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: Optional[float] = None
    """Maximum allowed value (inclusive)"""

    min: Optional[float] = None
    """Minimum allowed value (inclusive)"""


class InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponse(BaseModel):
    """Filter molecules by RDKit molecular descriptors.

    Each descriptor is constrained to a min/max range. Only descriptors you provide are checked — omitted descriptors are unconstrained.
    """

    type: Literal["rdkit_descriptor_filter"]

    fraction_csp3: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseFractionCsp3] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    mol_logp: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolLogp] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    mol_wt: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseMolWt] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_aromatic_rings: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumAromaticRings] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_h_acceptors: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHAcceptors] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_h_donors: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHDonors] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_heteroatoms: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumHeteroatoms] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_rings: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRings] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_rotatable_bonds: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseNumRotatableBonds] = None
    """Min/max range constraint for an RDKit molecular descriptor"""

    tpsa: Optional[InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponseTpsa] = None
    """Min/max range constraint for an RDKit molecular descriptor"""


class InputMoleculeFiltersCustomFilterSmartsCustomFilterResponse(BaseModel):
    """Filter molecules by custom SMARTS patterns.

    Molecules matching any pattern are rejected.
    """

    patterns: List[str]
    """SMARTS patterns. Molecules matching any pattern are rejected."""

    type: Literal["smarts_custom_filter"]


class InputMoleculeFiltersCustomFilterSmartsCatalogFilterResponse(BaseModel):
    """Filter molecules using a predefined SMARTS catalog of structural alerts."""

    catalog: Literal[
        "PAINS",
        "PAINS_A",
        "PAINS_B",
        "PAINS_C",
        "BRENK",
        "CHEMBL",
        "CHEMBL_BMS",
        "CHEMBL_Dundee",
        "CHEMBL_Glaxo",
        "CHEMBL_Inpharmatica",
        "CHEMBL_LINT",
        "CHEMBL_MLSMR",
        "CHEMBL_SureChEMBL",
        "NIH",
    ]
    """Predefined SMARTS catalog to apply.

    PAINS, BRENK, ChEMBL, and NIH catalogs reject known problematic substructures.
    """

    type: Literal["smarts_catalog_filter"]


class InputMoleculeFiltersCustomFilterSmilesRegexFilterResponse(BaseModel):
    """Filter molecules by regex patterns on their SMILES representation."""

    patterns: List[str]
    """Regex patterns applied to SMILES strings.

    Molecules matching any pattern are rejected.
    """

    type: Literal["smiles_regex_filter"]


InputMoleculeFiltersCustomFilter: TypeAlias = Union[
    InputMoleculeFiltersCustomFilterLipinskiFilterResponse,
    InputMoleculeFiltersCustomFilterRdkitDescriptorFilterResponse,
    InputMoleculeFiltersCustomFilterSmartsCustomFilterResponse,
    InputMoleculeFiltersCustomFilterSmartsCatalogFilterResponse,
    InputMoleculeFiltersCustomFilterSmilesRegexFilterResponse,
]


class InputMoleculeFilters(BaseModel):
    """Molecule filtering configuration.

    Controls both Boltz built-in SMARTS filtering and custom filters.
    """

    boltz_smarts_catalog_filter_level: Optional[Literal["recommended", "extra", "aggressive", "disabled"]] = None
    """
    Controls the stringency of Boltz's built-in SMARTS structural alert filtering,
    which removes molecules matching known problematic substructures. 'recommended'
    (default): applies a curated set of alerts balancing safety and hit rate.
    'extra': adds additional alerts beyond the recommended set for stricter
    filtering. 'aggressive': applies the most comprehensive alert set — may reject
    viable molecules. 'disabled': turns off Boltz SMARTS filtering entirely; only
    custom_filters will be applied.
    """

    custom_filters: Optional[List[InputMoleculeFiltersCustomFilter]] = None
    """Custom filters to apply. Molecules must pass all filters (AND logic)."""


class Input(BaseModel):
    num_molecules: int
    """Number of molecules to generate"""

    target: InputTarget
    """Target protein with binding pocket for small molecule design or screening"""

    chemical_space: Optional[Literal["enamine_real"]] = None
    """Chemical space to constrain generated molecules.

    Currently only 'enamine_real' (Enamine REAL chemical space) is supported.
    Additional options may be added in the future.
    """

    idempotency_key: Optional[str] = None
    """Client-provided key to prevent duplicate submissions on retries"""

    molecule_filters: Optional[InputMoleculeFilters] = None
    """Molecule filtering configuration.

    Controls both Boltz built-in SMARTS filtering and custom filters.
    """

    workspace_id: Optional[str] = None
    """Target workspace ID (admin keys only; ignored for workspace keys)"""


class Progress(BaseModel):
    num_molecules_generated: int
    """Number of molecules generated so far"""

    total_molecules_to_generate: int
    """Total number of molecules requested"""

    latest_result_id: Optional[str] = None
    """ID of the most recently generated result"""


class DesignStopResponse(BaseModel):
    """A small molecule design engine run that generates novel molecules"""

    id: str
    """Unique SmDesignRun identifier"""

    completed_at: Optional[datetime] = None

    created_at: datetime

    data_deleted_at: Optional[datetime] = None
    """When the input, output, and result data was permanently deleted.

    Null if data has not been deleted.
    """

    engine: Literal["boltz-sm-design"]
    """Engine used for small molecule design"""

    engine_version: str
    """Engine version used for small molecule design"""

    error: Optional[Error] = None

    input: Input

    livemode: bool
    """Whether this resource was created with a live API key."""

    progress: Optional[Progress] = None

    started_at: Optional[datetime] = None

    status: Literal["pending", "running", "succeeded", "failed", "stopped"]

    stopped_at: Optional[datetime] = None

    workspace_id: str
    """Workspace ID"""

    idempotency_key: Optional[str] = None
    """Client-provided idempotency key"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .usage_list_params import UsageListParams as UsageListParams
from .api_key_list_params import APIKeyListParams as APIKeyListParams
from .usage_list_response import UsageListResponse as UsageListResponse
from .api_key_create_params import APIKeyCreateParams as APIKeyCreateParams
from .api_key_list_response import APIKeyListResponse as APIKeyListResponse
from .workspace_list_params import WorkspaceListParams as WorkspaceListParams
from .api_key_create_response import APIKeyCreateResponse as APIKeyCreateResponse
from .api_key_revoke_response import APIKeyRevokeResponse as APIKeyRevokeResponse
from .workspace_create_params import WorkspaceCreateParams as WorkspaceCreateParams
from .workspace_list_response import WorkspaceListResponse as WorkspaceListResponse
from .workspace_update_params import WorkspaceUpdateParams as WorkspaceUpdateParams
from .workspace_create_response import WorkspaceCreateResponse as WorkspaceCreateResponse
from .workspace_update_response import WorkspaceUpdateResponse as WorkspaceUpdateResponse
from .workspace_archive_response import WorkspaceArchiveResponse as WorkspaceArchiveResponse
from .workspace_retrieve_response import WorkspaceRetrieveResponse as WorkspaceRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncOpaqueCursorPage, AsyncOpaqueCursorPage
from ...types.admin import usage_list_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.admin.usage_list_response import UsageListResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    """Retrieve aggregated usage data for the organization.

    Usage can be grouped by workspace and/or application, and filtered by time range, workspace, and application.
    """

    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_at: Union[str, datetime],
        starting_at: Union[str, datetime],
        window_size: Literal["HOUR", "DAY"],
        applications: Union[
            Literal[
                "structure_and_binding",
                "small_molecule_design",
                "small_molecule_library_screen",
                "protein_design",
                "protein_library_screen",
            ],
            List[
                Literal[
                    "structure_and_binding",
                    "small_molecule_design",
                    "small_molecule_library_screen",
                    "protein_design",
                    "protein_library_screen",
                ]
            ],
        ]
        | Omit = omit,
        group_by: Union[Literal["workspace_id", "application"], List[Literal["workspace_id", "application"]]]
        | Omit = omit,
        limit: int | Omit = omit,
        page: str | Omit = omit,
        workspace_ids: Union[str, SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncOpaqueCursorPage[UsageListResponse]:
        """
        Retrieve aggregated usage data across the organization, optionally grouped by
        workspace and/or application.

        Args:
          ending_at: End of the time range (ISO 8601)

          starting_at: Start of the time range (ISO 8601)

          applications: Filter to specific applications

          group_by: Group results by workspace_id and/or application

          limit: Maximum number of buckets to return

          page: Cursor for pagination

          workspace_ids: Filter to specific workspace IDs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/admin/usage",
            page=SyncOpaqueCursorPage[UsageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_at": ending_at,
                        "starting_at": starting_at,
                        "window_size": window_size,
                        "applications": applications,
                        "group_by": group_by,
                        "limit": limit,
                        "page": page,
                        "workspace_ids": workspace_ids,
                    },
                    usage_list_params.UsageListParams,
                ),
            ),
            model=UsageListResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    """Retrieve aggregated usage data for the organization.

    Usage can be grouped by workspace and/or application, and filtered by time range, workspace, and application.
    """

    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        ending_at: Union[str, datetime],
        starting_at: Union[str, datetime],
        window_size: Literal["HOUR", "DAY"],
        applications: Union[
            Literal[
                "structure_and_binding",
                "small_molecule_design",
                "small_molecule_library_screen",
                "protein_design",
                "protein_library_screen",
            ],
            List[
                Literal[
                    "structure_and_binding",
                    "small_molecule_design",
                    "small_molecule_library_screen",
                    "protein_design",
                    "protein_library_screen",
                ]
            ],
        ]
        | Omit = omit,
        group_by: Union[Literal["workspace_id", "application"], List[Literal["workspace_id", "application"]]]
        | Omit = omit,
        limit: int | Omit = omit,
        page: str | Omit = omit,
        workspace_ids: Union[str, SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UsageListResponse, AsyncOpaqueCursorPage[UsageListResponse]]:
        """
        Retrieve aggregated usage data across the organization, optionally grouped by
        workspace and/or application.

        Args:
          ending_at: End of the time range (ISO 8601)

          starting_at: Start of the time range (ISO 8601)

          applications: Filter to specific applications

          group_by: Group results by workspace_id and/or application

          limit: Maximum number of buckets to return

          page: Cursor for pagination

          workspace_ids: Filter to specific workspace IDs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/admin/usage",
            page=AsyncOpaqueCursorPage[UsageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_at": ending_at,
                        "starting_at": starting_at,
                        "window_size": window_size,
                        "applications": applications,
                        "group_by": group_by,
                        "limit": limit,
                        "page": page,
                        "workspace_ids": workspace_ids,
                    },
                    usage_list_params.UsageListParams,
                ),
            ),
            model=UsageListResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.list = to_raw_response_wrapper(
            usage.list,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.list = async_to_raw_response_wrapper(
            usage.list,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.list = to_streamed_response_wrapper(
            usage.list,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.list = async_to_streamed_response_wrapper(
            usage.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WorkspaceArchiveResponse", "DataRetention"]


class DataRetention(BaseModel):
    """How long result data is retained before automatic deletion.

    Defaults to 7 days. Maximum retention is 14 days (336 hours).
    """

    unit: Literal["hours", "days"]
    """Time unit for retention duration"""

    value: int
    """Duration value. Maximum retention is 14 days (or 336 hours)."""


class WorkspaceArchiveResponse(BaseModel):
    id: str
    """Workspace ID"""

    archived_at: Optional[datetime] = None

    created_at: datetime

    data_retention: DataRetention
    """How long result data is retained before automatic deletion.

    Defaults to 7 days. Maximum retention is 14 days (336 hours).
    """

    is_default: bool
    """Whether this is the default workspace"""

    name: Optional[str] = None
    """Workspace name"""

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DesignListResultsParams"]


class DesignListResultsParams(TypedDict, total=False):
    after_id: str
    """Return results after this ID"""

    before_id: str
    """Return results before this ID"""

    limit: int
    """Max results to return. Defaults to 100."""

    workspace_id: str
    """Workspace ID.

    Only used with admin API keys. Ignored (or validated) for workspace-scoped keys.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "LibraryScreenStartParams",
    "Molecule",
    "Target",
    "TargetEntity",
    "TargetEntityModification",
    "TargetEntityModificationCcdModification",
    "TargetEntityModificationSmilesModification",
    "MoleculeFilters",
    "MoleculeFiltersCustomFilter",
    "MoleculeFiltersCustomFilterLipinskiFilter",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilter",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterFractionCsp3",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterMolLogp",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterMolWt",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumAromaticRings",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHAcceptors",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHDonors",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHeteroatoms",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRings",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRotatableBonds",
    "MoleculeFiltersCustomFilterRdkitDescriptorFilterTpsa",
    "MoleculeFiltersCustomFilterSmartsCustomFilter",
    "MoleculeFiltersCustomFilterSmartsCatalogFilter",
    "MoleculeFiltersCustomFilterSmilesRegexFilter",
]


class LibraryScreenStartParams(TypedDict, total=False):
    molecules: Required[Iterable[Molecule]]
    """List of small molecules to screen."""

    target: Required[Target]
    """Target protein with binding pocket for small molecule design or screening"""

    idempotency_key: str
    """Client-provided key to prevent duplicate submissions on retries"""

    molecule_filters: MoleculeFilters
    """Molecule filtering configuration.

    Controls both Boltz built-in SMARTS filtering and custom filters.
    """

    workspace_id: str
    """Target workspace ID (admin keys only; ignored for workspace keys)"""


class Molecule(TypedDict, total=False):
    """A small molecule with SMILES and optional ID"""

    smiles: Required[str]
    """SMILES string of the molecule"""

    id: str
    """Optional identifier for this molecule"""


class TargetEntityModificationCcdModification(TypedDict, total=False):
    residue_index: Required[int]
    """0-based index of the residue to modify"""

    type: Required[Literal["ccd"]]

    value: Required[str]
    """CCD code from RCSB PDB (e.g.

    'MSE' for selenomethionine, 'SEP' for phosphoserine)
    """


class TargetEntityModificationSmilesModification(TypedDict, total=False):
    residue_index: Required[int]
    """0-based index of the residue to modify"""

    type: Required[Literal["smiles"]]

    value: Required[str]
    """SMILES string for the modification"""


TargetEntityModification: TypeAlias = Union[
    TargetEntityModificationCcdModification, TargetEntityModificationSmilesModification
]


class TargetEntity(TypedDict, total=False):
    chain_ids: Required[SequenceNotStr[str]]
    """Chain IDs for this entity"""

    modifications: Required[Iterable[TargetEntityModification]]
    """Post-translational modifications"""

    type: Required[Literal["protein"]]

    value: Required[str]
    """Amino acid sequence (one-letter codes)"""

    cyclic: bool
    """Whether the sequence is cyclic"""


class Target(TypedDict, total=False):
    """Target protein with binding pocket for small molecule design or screening"""

    entities: Required[Iterable[TargetEntity]]
    """Protein entities defining the target structure.

    Each entity represents a protein chain.
    """

    pocket_residues: Dict[str, Iterable[int]]
    """Binding pocket residues, keyed by chain ID.

    Each key is a chain ID (e.g. "A") and the value is an array of 0-indexed residue
    indices that define the binding pocket on that chain. When provided, these
    residues guide pocket extraction. When omitted, the model auto-detects the
    pocket.
    """

    reference_ligands: SequenceNotStr[str]
    """
    Reference ligands as SMILES strings that help the model identify the binding
    pocket. When omitted, a set of drug-like default ligands is used for pocket
    detection.
    """


class MoleculeFiltersCustomFilterLipinskiFilter(TypedDict, total=False):
    """Lipinski's Rule of Five filter.

    Rejects molecules that violate drug-likeness criteria based on molecular weight, LogP, hydrogen bond donors, and hydrogen bond acceptors.
    """

    max_hba: Required[float]
    """Maximum number of hydrogen bond acceptors. Lipinski threshold: 10"""

    max_hbd: Required[float]
    """Maximum number of hydrogen bond donors. Lipinski threshold: 5"""

    max_logp: Required[float]
    """Maximum LogP. Lipinski threshold: 5"""

    max_mw: Required[float]
    """Maximum molecular weight (Da). Lipinski threshold: 500"""

    type: Required[Literal["lipinski_filter"]]

    allow_single_violation: bool
    """If true, one rule violation is allowed (classic Rule of Five).

    Defaults to false (all rules must pass).
    """


class MoleculeFiltersCustomFilterRdkitDescriptorFilterFractionCsp3(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterMolLogp(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterMolWt(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumAromaticRings(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHAcceptors(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHDonors(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHeteroatoms(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRings(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRotatableBonds(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilterTpsa(TypedDict, total=False):
    """Min/max range constraint for an RDKit molecular descriptor"""

    max: float
    """Maximum allowed value (inclusive)"""

    min: float
    """Minimum allowed value (inclusive)"""


class MoleculeFiltersCustomFilterRdkitDescriptorFilter(TypedDict, total=False):
    """Filter molecules by RDKit molecular descriptors.

    Each descriptor is constrained to a min/max range. Only descriptors you provide are checked — omitted descriptors are unconstrained.
    """

    type: Required[Literal["rdkit_descriptor_filter"]]

    fraction_csp3: MoleculeFiltersCustomFilterRdkitDescriptorFilterFractionCsp3
    """Min/max range constraint for an RDKit molecular descriptor"""

    mol_logp: MoleculeFiltersCustomFilterRdkitDescriptorFilterMolLogp
    """Min/max range constraint for an RDKit molecular descriptor"""

    mol_wt: MoleculeFiltersCustomFilterRdkitDescriptorFilterMolWt
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_aromatic_rings: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumAromaticRings
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_h_acceptors: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHAcceptors
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_h_donors: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHDonors
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_heteroatoms: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumHeteroatoms
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_rings: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRings
    """Min/max range constraint for an RDKit molecular descriptor"""

    num_rotatable_bonds: MoleculeFiltersCustomFilterRdkitDescriptorFilterNumRotatableBonds
    """Min/max range constraint for an RDKit molecular descriptor"""

    tpsa: MoleculeFiltersCustomFilterRdkitDescriptorFilterTpsa
    """Min/max range constraint for an RDKit molecular descriptor"""


class MoleculeFiltersCustomFilterSmartsCustomFilter(TypedDict, total=False):
    """Filter molecules by custom SMARTS patterns.

    Molecules matching any pattern are rejected.
    """

    patterns: Required[SequenceNotStr[str]]
    """SMARTS patterns. Molecules matching any pattern are rejected."""

    type: Required[Literal["smarts_custom_filter"]]


class MoleculeFiltersCustomFilterSmartsCatalogFilter(TypedDict, total=False):
    """Filter molecules using a predefined SMARTS catalog of structural alerts."""

    catalog: Required[
        Literal[
            "PAINS",
            "PAINS_A",
            "PAINS_B",
            "PAINS_C",
            "BRENK",
            "CHEMBL",
            "CHEMBL_BMS",
            "CHEMBL_Dundee",
            "CHEMBL_Glaxo",
            "CHEMBL_Inpharmatica",
            "CHEMBL_LINT",
            "CHEMBL_MLSMR",
            "CHEMBL_SureChEMBL",
            "NIH",
        ]
    ]
    """Predefined SMARTS catalog to apply.

    PAINS, BRENK, ChEMBL, and NIH catalogs reject known problematic substructures.
    """

    type: Required[Literal["smarts_catalog_filter"]]


class MoleculeFiltersCustomFilterSmilesRegexFilter(TypedDict, total=False):
    """Filter molecules by regex patterns on their SMILES representation."""

    patterns: Required[SequenceNotStr[str]]
    """Regex patterns applied to SMILES strings.

    Molecules matching any pattern are rejected.
    """

    type: Required[Literal["smiles_regex_filter"]]


MoleculeFiltersCustomFilter: TypeAlias = Union[
    MoleculeFiltersCustomFilterLipinskiFilter,
    MoleculeFiltersCustomFilterRdkitDescriptorFilter,
    MoleculeFiltersCustomFilterSmartsCustomFilter,
    MoleculeFiltersCustomFilterSmartsCatalogFilter,
    MoleculeFiltersCustomFilterSmilesRegexFilter,
]


class MoleculeFilters(TypedDict, total=False):
    """Molecule filtering configuration.

    Controls both Boltz built-in SMARTS filtering and custom filters.
    """

    boltz_smarts_catalog_filter_level: Literal["recommended", "extra", "aggressive", "disabled"]
    """
    Controls the stringency of Boltz's built-in SMARTS structural alert filtering,
    which removes molecules matching known problematic substructures. 'recommended'
    (default): applies a curated set of alerts balancing safety and hit rate.
    'extra': adds additional alerts beyond the recommended set for stricter
    filtering. 'aggressive': applies the most comprehensive alert set — may reject
    viable molecules. 'disabled': turns off Boltz SMARTS filtering entirely; only
    custom_filters will be applied.
    """

    custom_filters: Iterable[MoleculeFiltersCustomFilter]
    """Custom filters to apply. Molecules must pass all filters (AND logic)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._models import SecurityOptions
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import admin, protein, predictions, small_molecule
    from .resources.admin.admin import AdminResource, AsyncAdminResource
    from .resources.protein.protein import ProteinResource, AsyncProteinResource
    from .resources.predictions.predictions import PredictionsResource, AsyncPredictionsResource
    from .resources.small_molecule.small_molecule import SmallMoleculeResource, AsyncSmallMoleculeResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "BoltzCompute",
    "AsyncBoltzCompute",
    "Client",
    "AsyncClient",
]


class BoltzCompute(SyncAPIClient):
    # client options
    api_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous BoltzCompute client instance.

        This automatically infers the `api_key` argument from the `BOLTZ_COMPUTE_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("BOLTZ_COMPUTE_API_KEY")
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("BOLTZ_COMPUTE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.boltz.bio"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def predictions(self) -> PredictionsResource:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import PredictionsResource

        return PredictionsResource(self)

    @cached_property
    def small_molecule(self) -> SmallMoleculeResource:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import SmallMoleculeResource

        return SmallMoleculeResource(self)

    @cached_property
    def protein(self) -> ProteinResource:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import ProteinResource

        return ProteinResource(self)

    @cached_property
    def admin(self) -> AdminResource:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AdminResource

        return AdminResource(self)

    @cached_property
    def with_raw_response(self) -> BoltzComputeWithRawResponse:
        return BoltzComputeWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BoltzComputeWithStreamedResponse:
        return BoltzComputeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key_auth if security.get("api_key_auth", False) else {}),
        }

    @property
    def _api_key_auth(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if headers.get("x-api-key") or isinstance(custom_headers.get("x-api-key"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the api_key to be set. Or for the `x-api-key` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncBoltzCompute(AsyncAPIClient):
    # client options
    api_key: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncBoltzCompute client instance.

        This automatically infers the `api_key` argument from the `BOLTZ_COMPUTE_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("BOLTZ_COMPUTE_API_KEY")
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("BOLTZ_COMPUTE_BASE_URL")
        if base_url is None:
            base_url = f"https://api.boltz.bio"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def predictions(self) -> AsyncPredictionsResource:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import AsyncPredictionsResource

        return AsyncPredictionsResource(self)

    @cached_property
    def small_molecule(self) -> AsyncSmallMoleculeResource:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import AsyncSmallMoleculeResource

        return AsyncSmallMoleculeResource(self)

    @cached_property
    def protein(self) -> AsyncProteinResource:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import AsyncProteinResource

        return AsyncProteinResource(self)

    @cached_property
    def admin(self) -> AsyncAdminResource:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AsyncAdminResource

        return AsyncAdminResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncBoltzComputeWithRawResponse:
        return AsyncBoltzComputeWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBoltzComputeWithStreamedResponse:
        return AsyncBoltzComputeWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._api_key_auth if security.get("api_key_auth", False) else {}),
        }

    @property
    def _api_key_auth(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if headers.get("x-api-key") or isinstance(custom_headers.get("x-api-key"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the api_key to be set. Or for the `x-api-key` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class BoltzComputeWithRawResponse:
    _client: BoltzCompute

    def __init__(self, client: BoltzCompute) -> None:
        self._client = client

    @cached_property
    def predictions(self) -> predictions.PredictionsResourceWithRawResponse:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import PredictionsResourceWithRawResponse

        return PredictionsResourceWithRawResponse(self._client.predictions)

    @cached_property
    def small_molecule(self) -> small_molecule.SmallMoleculeResourceWithRawResponse:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import SmallMoleculeResourceWithRawResponse

        return SmallMoleculeResourceWithRawResponse(self._client.small_molecule)

    @cached_property
    def protein(self) -> protein.ProteinResourceWithRawResponse:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import ProteinResourceWithRawResponse

        return ProteinResourceWithRawResponse(self._client.protein)

    @cached_property
    def admin(self) -> admin.AdminResourceWithRawResponse:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AdminResourceWithRawResponse

        return AdminResourceWithRawResponse(self._client.admin)


class AsyncBoltzComputeWithRawResponse:
    _client: AsyncBoltzCompute

    def __init__(self, client: AsyncBoltzCompute) -> None:
        self._client = client

    @cached_property
    def predictions(self) -> predictions.AsyncPredictionsResourceWithRawResponse:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import AsyncPredictionsResourceWithRawResponse

        return AsyncPredictionsResourceWithRawResponse(self._client.predictions)

    @cached_property
    def small_molecule(self) -> small_molecule.AsyncSmallMoleculeResourceWithRawResponse:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import AsyncSmallMoleculeResourceWithRawResponse

        return AsyncSmallMoleculeResourceWithRawResponse(self._client.small_molecule)

    @cached_property
    def protein(self) -> protein.AsyncProteinResourceWithRawResponse:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import AsyncProteinResourceWithRawResponse

        return AsyncProteinResourceWithRawResponse(self._client.protein)

    @cached_property
    def admin(self) -> admin.AsyncAdminResourceWithRawResponse:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AsyncAdminResourceWithRawResponse

        return AsyncAdminResourceWithRawResponse(self._client.admin)


class BoltzComputeWithStreamedResponse:
    _client: BoltzCompute

    def __init__(self, client: BoltzCompute) -> None:
        self._client = client

    @cached_property
    def predictions(self) -> predictions.PredictionsResourceWithStreamingResponse:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import PredictionsResourceWithStreamingResponse

        return PredictionsResourceWithStreamingResponse(self._client.predictions)

    @cached_property
    def small_molecule(self) -> small_molecule.SmallMoleculeResourceWithStreamingResponse:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import SmallMoleculeResourceWithStreamingResponse

        return SmallMoleculeResourceWithStreamingResponse(self._client.small_molecule)

    @cached_property
    def protein(self) -> protein.ProteinResourceWithStreamingResponse:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import ProteinResourceWithStreamingResponse

        return ProteinResourceWithStreamingResponse(self._client.protein)

    @cached_property
    def admin(self) -> admin.AdminResourceWithStreamingResponse:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AdminResourceWithStreamingResponse

        return AdminResourceWithStreamingResponse(self._client.admin)


class AsyncBoltzComputeWithStreamedResponse:
    _client: AsyncBoltzCompute

    def __init__(self, client: AsyncBoltzCompute) -> None:
        self._client = client

    @cached_property
    def predictions(self) -> predictions.AsyncPredictionsResourceWithStreamingResponse:
        """Run prediction models on molecular inputs.

        Each application is available as its own endpoint with application-specific inputs and outputs.
        """
        from .resources.predictions import AsyncPredictionsResourceWithStreamingResponse

        return AsyncPredictionsResourceWithStreamingResponse(self._client.predictions)

    @cached_property
    def small_molecule(self) -> small_molecule.AsyncSmallMoleculeResourceWithStreamingResponse:
        """
        Small Molecule Engine — design novel small molecules and screen compound libraries against protein targets. Includes de novo generation and virtual screening.
        """
        from .resources.small_molecule import AsyncSmallMoleculeResourceWithStreamingResponse

        return AsyncSmallMoleculeResourceWithStreamingResponse(self._client.small_molecule)

    @cached_property
    def protein(self) -> protein.AsyncProteinResourceWithStreamingResponse:
        """
        Protein Engine — design novel protein binders and screen protein libraries against targets. Includes de novo protein design and library screening.
        """
        from .resources.protein import AsyncProteinResourceWithStreamingResponse

        return AsyncProteinResourceWithStreamingResponse(self._client.protein)

    @cached_property
    def admin(self) -> admin.AsyncAdminResourceWithStreamingResponse:
        """Manage workspaces and API keys.

        Requires an admin API key. Admin keys have full access to all management and compute operations across all workspaces in the organization.
        """
        from .resources.admin import AsyncAdminResourceWithStreamingResponse

        return AsyncAdminResourceWithStreamingResponse(self._client.admin)


Client = BoltzCompute

AsyncClient = AsyncBoltzCompute



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["APIKeyRevokeResponse"]


class APIKeyRevokeResponse(BaseModel):
    id: str
    """API key ID"""

    allowed_ips: List[str]
    """IP addresses allowed to use this key. An empty array means all IPs are allowed."""

    created_at: datetime

    expires_at: Optional[datetime] = None
    """When the key expires. Null if the key does not expire."""

    is_active: bool

    key_prefix: str
    """First 12 characters of the key"""

    key_type: Literal["admin", "workspace"]

    last_used_at: Optional[datetime] = None

    livemode: bool
    """Whether this is a live API key (false for test keys)."""

    name: str
    """API key name"""

    workspace_id: Optional[str] = None
    """Workspace ID if workspace-scoped"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ...types.admin import workspace_list_params, workspace_create_params, workspace_update_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.admin.workspace_list_response import WorkspaceListResponse
from ...types.admin.workspace_create_response import WorkspaceCreateResponse
from ...types.admin.workspace_update_response import WorkspaceUpdateResponse
from ...types.admin.workspace_archive_response import WorkspaceArchiveResponse
from ...types.admin.workspace_retrieve_response import WorkspaceRetrieveResponse

__all__ = ["WorkspacesResource", "AsyncWorkspacesResource"]


class WorkspacesResource(SyncAPIResource):
    """
    Workspaces provide isolated environments for organizing predictions and engine runs across teams, projects, or customers. Each workspace has independent data retention settings and can be associated with workspace API keys.
    """

    @cached_property
    def with_raw_response(self) -> WorkspacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return WorkspacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WorkspacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return WorkspacesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        data_retention: workspace_create_params.DataRetention,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceCreateResponse:
        """
        Create a workspace

        Args:
          data_retention: How long result data is retained before automatic deletion. Defaults to 7 days.
              Maximum retention is 14 days (336 hours).

          name: Workspace name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/compute/v1/admin/workspaces",
            body=maybe_transform(
                {
                    "data_retention": data_retention,
                    "name": name,
                },
                workspace_create_params.WorkspaceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceCreateResponse,
        )

    def retrieve(
        self,
        workspace_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceRetrieveResponse:
        """
        Get a workspace

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._get(
            path_template("/compute/v1/admin/workspaces/{workspace_id}", workspace_id=workspace_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceRetrieveResponse,
        )

    def update(
        self,
        workspace_id: str,
        *,
        data_retention: workspace_update_params.DataRetention | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceUpdateResponse:
        """
        Update a workspace

        Args:
          data_retention: How long result data is retained before automatic deletion. Defaults to 7 days.
              Maximum retention is 14 days (336 hours).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._post(
            path_template("/compute/v1/admin/workspaces/{workspace_id}", workspace_id=workspace_id),
            body=maybe_transform(
                {
                    "data_retention": data_retention,
                    "name": name,
                },
                workspace_update_params.WorkspaceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceUpdateResponse,
        )

    def list(
        self,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[WorkspaceListResponse]:
        """
        List workspaces

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max items to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/admin/workspaces",
            page=SyncCursorPage[WorkspaceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                    },
                    workspace_list_params.WorkspaceListParams,
                ),
            ),
            model=WorkspaceListResponse,
        )

    def archive(
        self,
        workspace_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceArchiveResponse:
        """Archives a workspace and deactivates all its API keys.

        This action is
        irreversible.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return self._post(
            path_template("/compute/v1/admin/workspaces/{workspace_id}/archive", workspace_id=workspace_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceArchiveResponse,
        )


class AsyncWorkspacesResource(AsyncAPIResource):
    """
    Workspaces provide isolated environments for organizing predictions and engine runs across teams, projects, or customers. Each workspace has independent data retention settings and can be associated with workspace API keys.
    """

    @cached_property
    def with_raw_response(self) -> AsyncWorkspacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWorkspacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWorkspacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return AsyncWorkspacesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        data_retention: workspace_create_params.DataRetention,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceCreateResponse:
        """
        Create a workspace

        Args:
          data_retention: How long result data is retained before automatic deletion. Defaults to 7 days.
              Maximum retention is 14 days (336 hours).

          name: Workspace name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/compute/v1/admin/workspaces",
            body=await async_maybe_transform(
                {
                    "data_retention": data_retention,
                    "name": name,
                },
                workspace_create_params.WorkspaceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceCreateResponse,
        )

    async def retrieve(
        self,
        workspace_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceRetrieveResponse:
        """
        Get a workspace

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._get(
            path_template("/compute/v1/admin/workspaces/{workspace_id}", workspace_id=workspace_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceRetrieveResponse,
        )

    async def update(
        self,
        workspace_id: str,
        *,
        data_retention: workspace_update_params.DataRetention | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceUpdateResponse:
        """
        Update a workspace

        Args:
          data_retention: How long result data is retained before automatic deletion. Defaults to 7 days.
              Maximum retention is 14 days (336 hours).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._post(
            path_template("/compute/v1/admin/workspaces/{workspace_id}", workspace_id=workspace_id),
            body=await async_maybe_transform(
                {
                    "data_retention": data_retention,
                    "name": name,
                },
                workspace_update_params.WorkspaceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceUpdateResponse,
        )

    def list(
        self,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WorkspaceListResponse, AsyncCursorPage[WorkspaceListResponse]]:
        """
        List workspaces

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max items to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/admin/workspaces",
            page=AsyncCursorPage[WorkspaceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                    },
                    workspace_list_params.WorkspaceListParams,
                ),
            ),
            model=WorkspaceListResponse,
        )

    async def archive(
        self,
        workspace_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkspaceArchiveResponse:
        """Archives a workspace and deactivates all its API keys.

        This action is
        irreversible.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not workspace_id:
            raise ValueError(f"Expected a non-empty value for `workspace_id` but received {workspace_id!r}")
        return await self._post(
            path_template("/compute/v1/admin/workspaces/{workspace_id}/archive", workspace_id=workspace_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WorkspaceArchiveResponse,
        )


class WorkspacesResourceWithRawResponse:
    def __init__(self, workspaces: WorkspacesResource) -> None:
        self._workspaces = workspaces

        self.create = to_raw_response_wrapper(
            workspaces.create,
        )
        self.retrieve = to_raw_response_wrapper(
            workspaces.retrieve,
        )
        self.update = to_raw_response_wrapper(
            workspaces.update,
        )
        self.list = to_raw_response_wrapper(
            workspaces.list,
        )
        self.archive = to_raw_response_wrapper(
            workspaces.archive,
        )


class AsyncWorkspacesResourceWithRawResponse:
    def __init__(self, workspaces: AsyncWorkspacesResource) -> None:
        self._workspaces = workspaces

        self.create = async_to_raw_response_wrapper(
            workspaces.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            workspaces.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            workspaces.update,
        )
        self.list = async_to_raw_response_wrapper(
            workspaces.list,
        )
        self.archive = async_to_raw_response_wrapper(
            workspaces.archive,
        )


class WorkspacesResourceWithStreamingResponse:
    def __init__(self, workspaces: WorkspacesResource) -> None:
        self._workspaces = workspaces

        self.create = to_streamed_response_wrapper(
            workspaces.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            workspaces.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            workspaces.update,
        )
        self.list = to_streamed_response_wrapper(
            workspaces.list,
        )
        self.archive = to_streamed_response_wrapper(
            workspaces.archive,
        )


class AsyncWorkspacesResourceWithStreamingResponse:
    def __init__(self, workspaces: AsyncWorkspacesResource) -> None:
        self._workspaces = workspaces

        self.create = async_to_streamed_response_wrapper(
            workspaces.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            workspaces.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            workspaces.update,
        )
        self.list = async_to_streamed_response_wrapper(
            workspaces.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            workspaces.archive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.protein import (
    design_list_params,
    design_start_params,
    design_retrieve_params,
    design_list_results_params,
    design_estimate_cost_params,
)
from ...types.protein.design_list_response import DesignListResponse
from ...types.protein.design_stop_response import DesignStopResponse
from ...types.protein.design_start_response import DesignStartResponse
from ...types.protein.design_retrieve_response import DesignRetrieveResponse
from ...types.protein.design_delete_data_response import DesignDeleteDataResponse
from ...types.protein.design_list_results_response import DesignListResultsResponse
from ...types.protein.design_estimate_cost_response import DesignEstimateCostResponse

__all__ = ["DesignResource", "AsyncDesignResource"]


class DesignResource(SyncAPIResource):
    """Generate novel protein binders optimized for binding to a target structure.

    Results are scored by binding confidence (likelihood of protein-protein interaction) and structure confidence.
    """

    @cached_property
    def with_raw_response(self) -> DesignResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return DesignResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DesignResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return DesignResourceWithStreamingResponse(self)

    def retrieve(
        self,
        run_id: str,
        *,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignRetrieveResponse:
        """
        Retrieve a design run by ID, including progress and status

        Args:
          workspace_id: Workspace ID. Only used with admin API keys. Ignored (or validated) for
              workspace-scoped keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            path_template("/compute/v1/protein/design/{run_id}", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"workspace_id": workspace_id}, design_retrieve_params.DesignRetrieveParams),
            ),
            cast_to=DesignRetrieveResponse,
        )

    def list(
        self,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[DesignListResponse]:
        """
        List protein design runs, optionally filtered by workspace

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max items to return. Defaults to 100.

          workspace_id: Filter by workspace ID. Only used with admin API keys. If not provided, defaults
              to the workspace associated with the API key, or the default workspace for admin
              keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/protein/design",
            page=SyncCursorPage[DesignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                        "workspace_id": workspace_id,
                    },
                    design_list_params.DesignListParams,
                ),
            ),
            model=DesignListResponse,
        )

    def delete_data(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignDeleteDataResponse:
        """
        Permanently delete the input, output, and result data associated with this
        design run. The design run record itself is retained with a `data_deleted_at`
        timestamp. This action is irreversible.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            path_template("/compute/v1/protein/design/{run_id}/delete-data", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignDeleteDataResponse,
        )

    def estimate_cost(
        self,
        *,
        binder_specification: design_estimate_cost_params.BinderSpecification,
        num_proteins: int,
        target: design_estimate_cost_params.Target,
        idempotency_key: str | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignEstimateCostResponse:
        """
        Estimate the cost of a protein design run without creating any resource or
        consuming GPU.

        Args:
          binder_specification: Binder specification starting from an existing 3D structure. Upload a CIF/PDB
              file and select which chains to include, which residues to keep, and which
              regions to redesign. Only chains included in chain_selection are part of the
              engine run.

          num_proteins: Number of protein designs to generate

          target: Target specification (structure template or template-free)

          idempotency_key: Client-provided key to prevent duplicate submissions on retries

          workspace_id: Target workspace ID (admin keys only; ignored for workspace keys)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/compute/v1/protein/design/estimate-cost",
            body=maybe_transform(
                {
                    "binder_specification": binder_specification,
                    "num_proteins": num_proteins,
                    "target": target,
                    "idempotency_key": idempotency_key,
                    "workspace_id": workspace_id,
                },
                design_estimate_cost_params.DesignEstimateCostParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignEstimateCostResponse,
        )

    def list_results(
        self,
        run_id: str,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[DesignListResultsResponse]:
        """
        Retrieve paginated results from a protein design run

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max results to return. Defaults to 100.

          workspace_id: Workspace ID. Only used with admin API keys. Ignored (or validated) for
              workspace-scoped keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            path_template("/compute/v1/protein/design/{run_id}/results", run_id=run_id),
            page=SyncCursorPage[DesignListResultsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                        "workspace_id": workspace_id,
                    },
                    design_list_results_params.DesignListResultsParams,
                ),
            ),
            model=DesignListResultsResponse,
        )

    def start(
        self,
        *,
        binder_specification: design_start_params.BinderSpecification,
        num_proteins: int,
        target: design_start_params.Target,
        idempotency_key: str | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignStartResponse:
        """
        Create a new design run that generates novel protein binder candidates

        Args:
          binder_specification: Binder specification starting from an existing 3D structure. Upload a CIF/PDB
              file and select which chains to include, which residues to keep, and which
              regions to redesign. Only chains included in chain_selection are part of the
              engine run.

          num_proteins: Number of protein designs to generate

          target: Target specification (structure template or template-free)

          idempotency_key: Client-provided key to prevent duplicate submissions on retries

          workspace_id: Target workspace ID (admin keys only; ignored for workspace keys)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/compute/v1/protein/design",
            body=maybe_transform(
                {
                    "binder_specification": binder_specification,
                    "num_proteins": num_proteins,
                    "target": target,
                    "idempotency_key": idempotency_key,
                    "workspace_id": workspace_id,
                },
                design_start_params.DesignStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignStartResponse,
        )

    def stop(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignStopResponse:
        """
        Stop an in-progress protein design run early

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            path_template("/compute/v1/protein/design/{run_id}/stop", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignStopResponse,
        )


class AsyncDesignResource(AsyncAPIResource):
    """Generate novel protein binders optimized for binding to a target structure.

    Results are scored by binding confidence (likelihood of protein-protein interaction) and structure confidence.
    """

    @cached_property
    def with_raw_response(self) -> AsyncDesignResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDesignResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDesignResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/boltz-bio/boltz-compute-api-python#with_streaming_response
        """
        return AsyncDesignResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        run_id: str,
        *,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignRetrieveResponse:
        """
        Retrieve a design run by ID, including progress and status

        Args:
          workspace_id: Workspace ID. Only used with admin API keys. Ignored (or validated) for
              workspace-scoped keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            path_template("/compute/v1/protein/design/{run_id}", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"workspace_id": workspace_id}, design_retrieve_params.DesignRetrieveParams
                ),
            ),
            cast_to=DesignRetrieveResponse,
        )

    def list(
        self,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DesignListResponse, AsyncCursorPage[DesignListResponse]]:
        """
        List protein design runs, optionally filtered by workspace

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max items to return. Defaults to 100.

          workspace_id: Filter by workspace ID. Only used with admin API keys. If not provided, defaults
              to the workspace associated with the API key, or the default workspace for admin
              keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/compute/v1/protein/design",
            page=AsyncCursorPage[DesignListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                        "workspace_id": workspace_id,
                    },
                    design_list_params.DesignListParams,
                ),
            ),
            model=DesignListResponse,
        )

    async def delete_data(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignDeleteDataResponse:
        """
        Permanently delete the input, output, and result data associated with this
        design run. The design run record itself is retained with a `data_deleted_at`
        timestamp. This action is irreversible.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            path_template("/compute/v1/protein/design/{run_id}/delete-data", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignDeleteDataResponse,
        )

    async def estimate_cost(
        self,
        *,
        binder_specification: design_estimate_cost_params.BinderSpecification,
        num_proteins: int,
        target: design_estimate_cost_params.Target,
        idempotency_key: str | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignEstimateCostResponse:
        """
        Estimate the cost of a protein design run without creating any resource or
        consuming GPU.

        Args:
          binder_specification: Binder specification starting from an existing 3D structure. Upload a CIF/PDB
              file and select which chains to include, which residues to keep, and which
              regions to redesign. Only chains included in chain_selection are part of the
              engine run.

          num_proteins: Number of protein designs to generate

          target: Target specification (structure template or template-free)

          idempotency_key: Client-provided key to prevent duplicate submissions on retries

          workspace_id: Target workspace ID (admin keys only; ignored for workspace keys)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/compute/v1/protein/design/estimate-cost",
            body=await async_maybe_transform(
                {
                    "binder_specification": binder_specification,
                    "num_proteins": num_proteins,
                    "target": target,
                    "idempotency_key": idempotency_key,
                    "workspace_id": workspace_id,
                },
                design_estimate_cost_params.DesignEstimateCostParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignEstimateCostResponse,
        )

    def list_results(
        self,
        run_id: str,
        *,
        after_id: str | Omit = omit,
        before_id: str | Omit = omit,
        limit: int | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DesignListResultsResponse, AsyncCursorPage[DesignListResultsResponse]]:
        """
        Retrieve paginated results from a protein design run

        Args:
          after_id: Return results after this ID

          before_id: Return results before this ID

          limit: Max results to return. Defaults to 100.

          workspace_id: Workspace ID. Only used with admin API keys. Ignored (or validated) for
              workspace-scoped keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            path_template("/compute/v1/protein/design/{run_id}/results", run_id=run_id),
            page=AsyncCursorPage[DesignListResultsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_id": after_id,
                        "before_id": before_id,
                        "limit": limit,
                        "workspace_id": workspace_id,
                    },
                    design_list_results_params.DesignListResultsParams,
                ),
            ),
            model=DesignListResultsResponse,
        )

    async def start(
        self,
        *,
        binder_specification: design_start_params.BinderSpecification,
        num_proteins: int,
        target: design_start_params.Target,
        idempotency_key: str | Omit = omit,
        workspace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignStartResponse:
        """
        Create a new design run that generates novel protein binder candidates

        Args:
          binder_specification: Binder specification starting from an existing 3D structure. Upload a CIF/PDB
              file and select which chains to include, which residues to keep, and which
              regions to redesign. Only chains included in chain_selection are part of the
              engine run.

          num_proteins: Number of protein designs to generate

          target: Target specification (structure template or template-free)

          idempotency_key: Client-provided key to prevent duplicate submissions on retries

          workspace_id: Target workspace ID (admin keys only; ignored for workspace keys)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/compute/v1/protein/design",
            body=await async_maybe_transform(
                {
                    "binder_specification": binder_specification,
                    "num_proteins": num_proteins,
                    "target": target,
                    "idempotency_key": idempotency_key,
                    "workspace_id": workspace_id,
                },
                design_start_params.DesignStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignStartResponse,
        )

    async def stop(
        self,
        run_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DesignStopResponse:
        """
        Stop an in-progress protein design run early

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            path_template("/compute/v1/protein/design/{run_id}/stop", run_id=run_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DesignStopResponse,
        )


class DesignResourceWithRawResponse:
    def __init__(self, design: DesignResource) -> None:
        self._design = design

        self.retrieve = to_raw_response_wrapper(
            design.retrieve,
        )
        self.list = to_raw_response_wrapper(
            design.list,
        )
        self.delete_data = to_raw_response_wrapper(
            design.delete_data,
        )
        self.estimate_cost = to_raw_response_wrapper(
            design.estimate_cost,
        )
        self.list_results = to_raw_response_wrapper(
            design.list_results,
        )
        self.start = to_raw_response_wrapper(
            design.start,
        )
        self.stop = to_raw_response_wrapper(
            design.stop,
        )


class AsyncDesignResourceWithRawResponse:
    def __init__(self, design: AsyncDesignResource) -> None:
        self._design = design

        self.retrieve = async_to_raw_response_wrapper(
            design.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            design.list,
        )
        self.delete_data = async_to_raw_response_wrapper(
            design.delete_data,
        )
        self.estimate_cost = async_to_raw_response_wrapper(
            design.estimate_cost,
        )
        self.list_results = async_to_raw_response_wrapper(
            design.list_results,
        )
        self.start = async_to_raw_response_wrapper(
            design.start,
        )
        self.stop = async_to_raw_response_wrapper(
            design.stop,
        )


class DesignResourceWithStreamingResponse:
    def __init__(self, design: DesignResource) -> None:
        self._design = design

        self.retrieve = to_streamed_response_wrapper(
            design.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            design.list,
        )
        self.delete_data = to_streamed_response_wrapper(
            design.delete_data,
        )
        self.estimate_cost = to_streamed_response_wrapper(
            design.estimate_cost,
        )
        self.list_results = to_streamed_response_wrapper(
            design.list_results,
        )
        self.start = to_streamed_response_wrapper(
            design.start,
        )
        self.stop = to_streamed_response_wrapper(
            design.stop,
        )


class AsyncDesignResourceWithStreamingResponse:
    def __init__(self, design: AsyncDesignResource) -> None:
        self._design = design

        self.retrieve = async_to_streamed_response_wrapper(
            design.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            design.list,
        )
        self.delete_data = async_to_streamed_response_wrapper(
            design.delete_data,
        )
        self.estimate_cost = async_to_streamed_response_wrapper(
            design.estimate_cost,
        )
        self.list_results = async_to_streamed_response_wrapper(
            design.list_results,
        )
        self.start = async_to_streamed_response_wrapper(
            design.start,
        )
        self.stop = async_to_streamed_response_wrapper(
            design.stop,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .design import (
    DesignResource,
    AsyncDesignResource,
    DesignResourceWithRawResponse,
    AsyncDesignResourceWithRawResponse,
    DesignResourceWithStreamingResponse,
    AsyncDesignResourceWithStreamingResponse,
)
from .protein import (
    ProteinResource,
    AsyncProteinResource,
    ProteinResourceWithRawResponse,
    AsyncProteinResourceWithRawResponse,
    ProteinResourceWithStreamingResponse,
    AsyncProteinResourceWithStreamingResponse,
)
from .library_screen import (
    LibraryScreenResource,
    AsyncLibraryScreenResource,
    LibraryScreenResourceWithRawResponse,
    AsyncLibraryScreenResourceWithRawResponse,
    LibraryScreenResourceWithStreamingResponse,
    AsyncLibraryScreenResourceWithStreamingResponse,
)

__all__ = [
    "DesignResource",
    "AsyncDesignResource",
    "DesignResourceWithRawResponse",
    "AsyncDesignResourceWithRawResponse",
    "DesignResourceWithStreamingResponse",
    "AsyncDesignResourceWithStreamingResponse",
    "LibraryScreenResource",
    "AsyncLibraryScreenResource",
    "LibraryScreenResourceWithRawResponse",
    "AsyncLibraryScreenResourceWithRawResponse",
    "LibraryScreenResourceWithStreamingResponse",
    "AsyncLibraryScreenResourceWithStreamingResponse",
    "ProteinResource",
    "AsyncProteinResource",
    "ProteinResourceWithRawResponse",
    "AsyncProteinResourceWithRawResponse",
    "ProteinResourceWithStreamingResponse",
    "AsyncProteinResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["APIKeyListParams"]


class APIKeyListParams(TypedDict, total=False):
    after_id: str
    """Return results after this ID"""

    before_id: str
    """Return results before this ID"""

    limit: int
    """Max items to return"""

    workspace_id: str
    """Filter by workspace ID. If not provided, returns keys across all workspaces."""
